@@ -3,7 +3,7 @@ from sly import Parser
 from mindsdb_sql.parser.ast import (ASTNode, Constant, Identifier, Select, BinaryOperation, UnaryOperation, Join,
                                     NullConstant,
                                     TypeCast, Tuple, OrderBy, Operation, Function, Parameter, BetweenOperation, Star,
-                                    Union, Use, CommonTableExpression)
+                                    Union, Use, Show, CommonTableExpression)
 from mindsdb_sql.exceptions import ParsingException
 from mindsdb_sql.parser.lexer import SQLLexer
 from mindsdb_sql.utils import ensure_select_keyword_order, JoinType
@@ -20,13 +20,55 @@ class SQLParser(Parser):
     )
 
     # Top-level statements
-
-    @_('use',
-       'select',
-       'union')
+    @_('show',
+       'union',
+       'select',)
     def query(self, p):
         return p[0]
 
+    # Show
+
+    @_('SHOW show_category show_condition_or_nothing')
+    def show(self, p):
+        condition = p.show_condition_or_nothing['condition'] if p.show_condition_or_nothing else None
+        expression = p.show_condition_or_nothing['expression'] if p.show_condition_or_nothing else None
+        return Show(category=p.show_category,
+                    condition=condition,
+                    expression=expression)
+
+    @_('show_condition_token expr',
+       'empty')
+    def show_condition_or_nothing(self, p):
+        if not p[0]:
+            return None
+        return dict(condition=p[0], expression=p[1])
+
+    @_('WHERE',
+       'FROM',
+       'LIKE')
+    def show_condition_token(self, p):
+        return p[0]
+
+    @_('SCHEMAS',
+       'DATABASES',
+       'TABLES',
+       'FULL TABLES',
+       'VARIABLES',
+       'SESSION VARIABLES',
+       'SESSION STATUS',
+       'GLOBAL VARIABLES',
+       'PROCEDURE STATUS',
+       'FUNCTION STATUS',
+       'INDEX',
+       'CREATE TABLE',
+       'WARNINGS',
+       'ENGINES',
+       'CHARSET',
+       'COLLATION',
+       'TABLE STATUS')
+    def show_category(self, p):
+        return ' '.join([x for x in p])
+
     # USE
 
     @_('USE identifier')
@@ -217,14 +259,15 @@ class SQLParser(Parser):
         return p.parameter
 
     @_('JOIN',
-       'LEFT_JOIN',
-       'RIGHT_JOIN',
-       'INNER_JOIN',
-       'FULL_JOIN',
-       'CROSS_JOIN',
-       'OUTER_JOIN')
+       'LEFT JOIN',
+       'RIGHT JOIN',
+       'INNER JOIN',
+       'FULL JOIN',
+       'CROSS JOIN',
+       'OUTER JOIN',
+       )
     def join_clause(self, p):
-        return p[0]
+        return ' '.join([x for x in p])
 
     @_('SELECT DISTINCT result_columns')
     def select(self, p):
